package com.example.data_push.util;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.google.gson.JsonArray;
import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import com.google.gson.JsonParser;
import org.dom4j.Attribute;
import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;

import java.util.List;
import java.util.Map.Entry;
import java.util.Set;

public class JsonToXml2 {


    private static final String STR_JSON = "{\"stakeapply\":{\"voltageLevel\":\"AC00062\",\"stakeList\":{\"stake\":[{\"stakeAssetNO\":45754745,\"otherStakeTypeRemark\":\"xxx\",\"stationId\":\"547547547547\"},{\"stakeAssetNO\":34325325322,\"otherStakeTypeRemark\":\"xxx\",\"stationId\":\"52354645462\"}]},\"otherStationTypeRemark\":\"xxx\",\"stationAddr\":\"哈哈\",\"custLists\":{\"custList\":{\"custId\":\"7547547547\",\"custPhone\":13666666666,\"contactMode\":1}},\"principalList\":{\"principal\":[{\"principalName\":121212,\"principalType\":1},{\"principalName\":12121233,\"principalType\":1}]}}}";

    /**
     * 将json字符串转换成xml
     *
     * @param json          json字符串
     * @param parentElement xml根节点
     */
    public static Element jsonToXml(String json, Element parentElement) {
        JsonObject jsonObject = JsonParser.parseString(json).getAsJsonObject();
        return toXml(jsonObject, parentElement, null);
    }

    /**
     * 将json字符串转换成xml
     *
     * @param jsonElement   待解析json对象元素
     * @param parentElement 上一层xml的dom对象
     * @param name          父节点
     */
    public static Element toXml(JsonElement jsonElement, Element parentElement, String name) {
        if (jsonElement instanceof JsonArray) {
            //是json数据，需继续解析
            JsonArray sonJsonArray = (JsonArray) jsonElement;
            for (int i = 0; i < sonJsonArray.size(); i++) {
                JsonElement arrayElement = sonJsonArray.get(i);
                toXml(arrayElement, parentElement, name);
            }
        } else if (jsonElement instanceof JsonObject) {
            //说明是一个json对象字符串，需要继续解析
            JsonObject sonJsonObject = (JsonObject) jsonElement;
            Element currentElement = null;
            if (name != null) {
                currentElement = parentElement.addElement(name);
            }
            Set<Entry<String, JsonElement>> set = sonJsonObject.entrySet();
            for (Entry<String, JsonElement> s : set) {
                toXml(s.getValue(), currentElement != null ? currentElement : parentElement, s.getKey());
            }
        } else {
            //说明是一个键值对的key,可以作为节点插入了
            addAttribute(parentElement, name, jsonElement.getAsString());
        }
        return parentElement;
    }

    /**
     * @param element 父节点
     * @param name    子节点的名字
     * @param value   子节点的值
     */
    public static void addAttribute(Element element, String name, String value) {
        //增加子节点，并为子节点赋值
        Element el = element.addElement(name);
        el.addText(value);

    }
    /**
     * 将xml转换为JSON对象
     *
     * @param xml xml字符串
     * @return
     * @throws Exception
     */
    public static JSONObject xmltoJson(String xml) throws Exception {
        JSONObject jsonObject = new JSONObject();
        Document document = DocumentHelper.parseText(xml);
        //获取根节点元素对象
        Element root = document.getRootElement();
        iterateNodes(root, jsonObject);
        return jsonObject;
    }

    /**
     * 遍历元素
     *
     * @param node 元素
     * @param json 将元素遍历完成之后放的JSON对象
     */
    @SuppressWarnings("unchecked")
    public static void iterateNodes(Element node, JSONObject json) {
        //获取当前元素的名称
        String nodeName = node.getName();
        //判断已遍历的JSON中是否已经有了该元素的名称
        if (json.containsKey(nodeName)) {
            //该元素在同级下有多个
            Object Object = json.get(nodeName);
            JSONArray array = null;
            if (Object instanceof JSONArray) {
                array = (JSONArray) Object;
            } else {
                array = new JSONArray();
                array.add(Object);
            }
            //获取该元素下所有子元素
            List<Element> listElement = node.elements();
            if (listElement.isEmpty()) {
                //该元素无子元素，获取元素的值
                String nodeValue = node.getTextTrim();
                array.add(nodeValue);
                json.put(nodeName, array);
                return;
            }
            //有子元素
            JSONObject newJson = new JSONObject();
            //遍历所有子元素
            for (Element e : listElement) {
                //递归
                iterateNodes(e, newJson);
            }
            List<Attribute> attss = node.attributes();
            // 获取book属性名和属性值
            for (Attribute att : attss) {
                newJson.put(att.getName(), att.getValue());
              //  System.out.println("节点名：" + att.getName() + "节点值：" + att.getValue());
            }
            array.add(newJson);
            json.put(nodeName, array);
            return;
        }
        //该元素同级下第一次遍历
        //获取该元素下所有子元素
        List<Element> listElement = node.elements();
        if (listElement.isEmpty()) {
            //该元素无子元素，获取元素的值
            String nodeValue = node.getTextTrim();
            json.put(nodeName, nodeValue);
            return;
        }
        //有子节点，新建一个JSONObject来存储该节点下子节点的值
        JSONObject object = new JSONObject();
        //遍历所有一级子节点
        for (Element e : listElement) {

            //递归
            iterateNodes(e, object);
        }
        List<Attribute> attss = node.attributes();
        // 获取book属性名和属性值
        for (Attribute att : attss) {
            object.put(att.getName(), att.getValue());
           // System.out.println("节点名：" + att.getName() + "节点值：" + att.getValue());
        }
        json.put(nodeName, object);
        return;
    }

/*    public static void main(String[] args) throws Exception {
        Document document = DocumentHelper.createDocument();
        Element root = document.addElement("data"); //默认根节点
        ZybContactHazardFactor contactHazardFactor=new ZybContactHazardFactor();
        contactHazardFactor.setHazardCode("331");
        ZybContactHazardFactor contactHazardFactor2=new ZybContactHazardFactor();
        contactHazardFactor2.setHazardCode("331");
        List<ZybContactHazardFactor>list=new ArrayList<>();
        list.add(contactHazardFactor);
        list.add(contactHazardFactor2);
        ZybReportCard zybReportCard=new ZybReportCard();
        Map<String,List<ZybContactHazardFactor>> listMap=new HashMap<>();
        listMap.put("contactHazardFactor",list);
        zybReportCard.setContactHazardFactorList(listMap);
        Element el = jsonToXml(new Gson().toJson(zybReportCard), root);
        System.out.println(el.asXML());
    }*/
    /*
    json串
    {
        "stakeapply": {
            "voltageLevel": "AC00062",
            "stakeList": {
                "stake": [
                    {
                        "stakeAssetNO": 45754745,
                        "otherStakeTypeRemark": "xxx",
                        "stationId": "547547547547"
                    },
                    {
                        "stakeAssetNO": 34325325322,
                        "otherStakeTypeRemark": "xxx",
                        "stationId": "52354645462"
                    }
                ]
            },
            "otherStationTypeRemark": "xxx",
            "stationAddr": "哈哈",
            "custLists": {
                "custList": {
                    "custId": "7547547547",
                    "custPhone": 13666666666,
                    "contactMode": 1
                }
            },
            "principalList": {
                "principal": [
                    {
                        "principalName": 121212,
                        "principalType": 1
                    },
                    {
                        "principalName": 12121233,
                        "principalType": 1
                    }
                ]
            }
        }
    }
    */

    /*
    转换后的xml
    <?xml version="1.0" encoding="UTF-8"?>

    <stakeapply>
      <voltageLevel>AC00062</voltageLevel>
      <stakeList>
        <stake>
          <stakeAssetNO>45754745</stakeAssetNO>
          <otherStakeTypeRemark>xxx</otherStakeTypeRemark>
          <stationId>547547547547</stationId>
        </stake>
        <stake>
          <stakeAssetNO>34325325322</stakeAssetNO>
          <otherStakeTypeRemark>xxx</otherStakeTypeRemark>
          <stationId>52354645462</stationId>
        </stake>
      </stakeList>
      <otherStationTypeRemark>xxx</otherStationTypeRemark>
      <stationAddr>哈哈</stationAddr>
      <custLists>
        <custList>
          <custId>7547547547</custId>
          <custPhone>13666666666</custPhone>
          <contactMode>1</contactMode>
        </custList>
      </custLists>
      <principalList>
        <principal>
          <principalName>121212</principalName>
          <principalType>1</principalType>
        </principal>
        <principal>
          <principalName>12121233</principalName>
          <principalType>1</principalType>
        </principal>
      </principalList>
    </stakeapply>
    */
}